Crate commonware_storage

Crate commonware_storage 

Persist and retrieve data from an abstract store.

Status

Stability varies by primitive. See README for details.

Re-exports

pub use merkle::mmb;
pub use merkle::mmr;

Modules

archive
A write-once key-value store for ordered data.
bmt
Stateless Binary Merkle Tree (BMT).
cache
A prunable cache for ordered data with index-based lookups.
freezer
An immutable key-value store optimized for minimal memory usage and write amplification.
index
Memory-efficient index structures for mapping translated keys to values.
journal
An append-only log for storing arbitrary data.
merkle
Shared types for Merkle-family data structures (MMR, MMB).
metadata
A key-value store optimized for atomically committing a small collection of metadata.
ordinal
A persistent index that maps sparse indices to commonware_utils::Arrays.
qmdb
A collection of authenticated databases inspired by QMDB (Quick Merkle Database).
queue
A durable, at-least-once delivery queue backed by a variable::Journal.
rmap
A collection that manages disjoint, inclusive ranges [start, end].
translator
Primitive implementations of Translator.

Structs

AuthenticatedBitMap
A bitmap supporting inclusion proofs through Merkelization.

Traits

Context
A runtime context providing storage, timing, and metrics capabilities.
Persistable
A storage structure with capabilities to persist and recover state across restarts.

Type Aliases

MerkleizedBitMap
A merkleized bitmap whose root digest has been computed and cached.
UnmerkleizedBitMap
An unmerkleized bitmap whose root digest has not been computed.