Module archive

Module archive 

A write-once key-value store for ordered data.

Archive is a key-value store designed for workloads where data is written only once and each item is addressed by both an index and a key. Workloads with unique indices should use Archive and workloads with overlapping indices should use MultiArchive (allows all items with the same index to be retrieved). The same key may be stored at multiple indices in either case, and a key lookup may return any of the associated values.

Modules

immutable
An immutable key-value store for ordered data with a minimal memory footprint.
prunable
A prunable key-value store for ordered data.

Enums

Error
Errors that can occur when interacting with the archive.
Identifier
Subject of a get or has operation.

Traits

Archive
A write-once key-value store addressed by both an index and a key.
MultiArchive
Extension of Archive that supports multiple items at the same index.