pub struct Freezer<E: BufferPooler + Context, K: Array, V: CodecShared> { /* private fields */ }Expand description
Implementation of Freezer.
Implementations
Sourceimpl<E: BufferPooler + Context, K: Array, V: CodecShared> Freezer<E, K, V>
impl<E: BufferPooler + Context, K: Array, V: CodecShared> Freezer<E, K, V>
Sourcepub async fn init(
context: E,
config: Config<V::Cfg>,
checkpoint: Option<Checkpoint>,
) -> Result<Self, Error>
pub async fn init( context: E, config: Config<V::Cfg>, checkpoint: Option<Checkpoint>, ) -> Result<Self, Error>
Initialize a Freezer instance, aligning existing data to a Checkpoint when provided.
Passing None or an empty Checkpoint deletes any existing freezer data and starts empty.
Sourcepub async fn put(&mut self, key: K, value: V) -> Result<Cursor, Error>
pub async fn put(&mut self, key: K, value: V) -> Result<Cursor, Error>
Put a key-value pair into the Freezer. If the key already exists, the value is updated.
Sourcepub async fn get<'a>(
&'a self,
identifier: Identifier<'a, K>,
) -> Result<Option<V>, Error>
pub async fn get<'a>( &'a self, identifier: Identifier<'a, K>, ) -> Result<Option<V>, Error>
Get the value for a given Identifier.
If a Cursor is known for the required key, it
is much faster to use it than searching for a key.
Sourcepub async fn has(&self, key: &K) -> Result<bool, Error>
pub async fn has(&self, key: &K) -> Result<bool, Error>
Check whether a value exists for a given key.
Walks the same key index chain as Self::get with Identifier::Key
but never reads values.
Sourcepub async fn sync(&mut self) -> Result<Checkpoint, Error>
pub async fn sync(&mut self) -> Result<Checkpoint, Error>
Sync all pending data in Freezer.
If the table needs to be resized, the resize will begin during this sync.
The resize operation is performed incrementally across multiple sync calls
to avoid a large latency spike (or unexpected long latency for Freezer::put).
Each sync will process up to table_resize_chunk_size entries until the resize
is complete.
Sourcepub async fn close(self) -> Result<Checkpoint, Error>
pub async fn close(self) -> Result<Checkpoint, Error>
Close the Freezer and return a Checkpoint for recovery.