BatchChain

Trait BatchChain 

pub trait BatchChain<Item> {
    // Required method
    fn collect(&self, into: &mut Vec<Arc<Vec<Item>>>);
}
A chain of batches whose items can be collected in append order.

Required Methods

fn collect(&self, into: &mut Vec<Arc<Vec<Item>>>)

Collect the items from the deepest ancestor batch up to and including the current batch in append order.

Implementors

Source

impl<'a, H: Hasher, P: Readable<H::Digest> + BatchChain<Item>, Item: Send + Sync> BatchChain<Item> for MerkleizedBatch<'a, H, P, Item>

Source

impl<E: Storage + Clock + Metrics, D: Digest, Item> BatchChain<Item> for Mmr<E, D>