pub struct Config {
pub partition: String,
pub items_per_blob: NonZeroU64,
pub page_cache: CacheRef,
pub write_buffer: NonZeroUsize,
}Expand description
Configuration for Journal storage.
Fields
partition: StringPrefix for the journal partitions.
Blobs are stored in partition (legacy) if it contains data, otherwise in
{partition}-blobs. Metadata is stored in {partition}-metadata.
items_per_blob: NonZeroU64The maximum number of journal items to store in each blob.
Retained non-tail blobs are expected to be full relative to their logical capacity. A mid-blob oldest blob may physically hold fewer than this many items, and the newest blob may contain fewer items.
page_cache: CacheRefThe page cache to use for caching data.
write_buffer: NonZeroUsizeThe size of the write buffer to use for each blob.