Module segmented

Module segmented 

Segmented journals with section-based storage.

This module provides journal implementations that organize data into sections, where each section is stored in a separate blob.

Modules

fixed
Segmented journal for fixed-size items.
glob
Simple section-based blob storage for values.
oversized
Segmented journal for oversized values.
variable
An append-only log for storing arbitrary variable length items.