Error

Enum Error 

pub enum Error<F: Family> {
Show 21 variants NonLeaf(Position<F>), PositionOverflow(Position<F>), LocationOverflow(Location<F>), Empty, RangeOutOfBounds(Location<F>), InvalidSize(u64), LeafOutOfBounds(Location<F>), ElementPruned(Position<F>), InvalidPinnedNodes, StaleBatch { expected: Position<F>, actual: Position<F>, }, AncestorDropped { expected: Position<F>, actual: Position<F>, }, InvalidProof, RootMismatch, MissingDigest(Position<F>), Metadata(Error), Journal(Error), Runtime(Error), MissingNode(Position<F>), DataCorrupted(&'static str), MissingGraftedLeaf(Position<F>), BitOutOfBounds(u64, u64),
}
Errors that can occur when interacting with a Merkle-family data structure.

Variants



NonLeaf(Position<F>)

The position does not correspond to a leaf node.



PositionOverflow(Position<F>)

The position exceeds the valid range.



LocationOverflow(Location<F>)

The location exceeds the valid range.



Empty

The range is empty but must contain at least one element.



RangeOutOfBounds(Location<F>)

The end of a range is out of bounds.



InvalidSize(u64)

The requested size is invalid.



LeafOutOfBounds(Location<F>)

A requested leaf location exceeds the current leaf count.



ElementPruned(Position<F>)

A required node was not available (e.g. pruned).



InvalidPinnedNodes

The provided pinned node list does not match the expected pruning boundary.



StaleBatch

Structure has diverged incompatibly from the batch’s ancestor chain.

Fields

expected: Position<F>

The base size when the batch chain was forked.

actual: Position<F>

The current structure size.



AncestorDropped

An ancestor batch was dropped before this batch was applied, causing data loss. All ancestors must be kept alive until descendants are applied.

Fields

expected: Position<F>

The expected size after applying all ancestors + this batch.

actual: Position<F>

The actual size (less than expected due to missing ancestor data).



InvalidProof

The proof is invalid.



RootMismatch

The root does not match the computed root.



MissingDigest(Position<F>)

A required digest is missing.



Metadata(Error)

A metadata error occurred.



Journal(Error)

A journal error occurred.



Runtime(Error)

A runtime error occurred.



MissingNode(Position<F>)

A required node is missing.



DataCorrupted(&'static str)

Data is corrupted.



MissingGraftedLeaf(Position<F>)

A required grafted leaf digest is missing.



BitOutOfBounds(u64, u64)

Bit offset is out of bounds.