pub struct Config<F: Family, D: Digest> {
pub nodes: Vec<D>,
pub pruning_boundary: Location<F>,
pub pinned_nodes: Vec<D>,
}Expand description
Configuration for initializing a Mem.
Fields
nodes: Vec<D>The retained nodes.
pruning_boundary: Location<F>The leaf location up to which pruning has been performed, or 0 if never pruned.
pinned_nodes: Vec<D>The pinned nodes, in the order expected by Family::nodes_to_pin.