pub struct Location<F: Family>(/* private fields */);Expand description
A Location is a leaf index or leaf count in a Merkle structure. This is in contrast to a Position, which is a node index or node count.
Limits
Values up to the family’s maximum are valid (see Location::is_valid). As a 0-based leaf
index, valid indices are 0..MAX - 1. As a leaf count or exclusive range-end, the maximum
is MAX itself.
Implementations
Sourceimpl<F: Family> Location<F>
impl<F: Family> Location<F>
Sourcepub const fn is_valid(self) -> bool
pub const fn is_valid(self) -> bool
Returns true iff this value is a valid leaf count (<= MAX_LEAVES).
Sourcepub const fn is_valid_index(self) -> bool
pub const fn is_valid_index(self) -> bool
Returns true iff this value is a valid 0-based leaf index (< MAX_LEAVES).
Sourcepub const fn checked_add(self, rhs: u64) -> Option<Self>
pub const fn checked_add(self, rhs: u64) -> Option<Self>
Return self + rhs returning None on overflow or if result exceeds the maximum.
Sourcepub const fn checked_sub(self, rhs: u64) -> Option<Self>
pub const fn checked_sub(self, rhs: u64) -> Option<Self>
Return self - rhs returning None on underflow.
Sourcepub const fn saturating_add(self, rhs: u64) -> Self
pub const fn saturating_add(self, rhs: u64) -> Self
Return self + rhs saturating at the maximum.
Sourcepub const fn saturating_sub(self, rhs: u64) -> Self
pub const fn saturating_sub(self, rhs: u64) -> Self
Return self - rhs saturating at zero.