Position

Struct Position 

pub struct Position<F: Family>(/* private fields */);
A Position is a node index or node count in a Merkle structure. This is in contrast to a Location, which is a leaf index or leaf count.

Limits

Values up to the family’s maximum are valid (see Position::is_valid). As a 0-based node index, valid indices are 0..MAX - 1. As a node count or total size, the maximum is MAX itself. Use Position::is_valid_size to ask whether a count is a structurally valid size for the specific Merkle family.

Implementations

Source

impl<F: Family> Position<F>

pub const fn new(pos: u64) -> Self

Return a new Position from a raw u64.

pub const fn as_u64(self) -> u64

Return the underlying u64 value.

pub const fn is_valid(self) -> bool

Returns true iff this value is a valid node count or size (<= MAX_NODES).

pub const fn is_valid_index(self) -> bool

Returns true iff this value is a valid 0-based node index (< MAX_NODES).

pub const fn checked_add(self, rhs: u64) -> Option<Self>

Return self + rhs returning None on overflow or if result exceeds the maximum.

pub const fn checked_sub(self, rhs: u64) -> Option<Self>

Return self - rhs returning None on underflow.

pub const fn saturating_add(self, rhs: u64) -> Self

Return self + rhs saturating at the maximum.

pub const fn saturating_sub(self, rhs: u64) -> Self

Return self - rhs saturating at zero.

pub fn is_valid_size(self) -> bool

Returns whether this is a valid size for this Merkle structure.

Methods from Deref<Target = u64>

1.43.0 · Source

pub const MIN: u64 = 0

1.43.0 · Source

pub const MAX: u64

1.53.0 · Source

pub const BITS: u32