pub struct Position(/* private fields */);Expand description
Implementations
Sourceimpl Position
impl Position
Sourcepub const fn is_valid(self) -> bool
pub const fn is_valid(self) -> bool
Returns true iff this value is within the valid range (<= MAX_POSITION).
This covers both node indices (< MAX_POSITION) and node counts (<= MAX_POSITION).
Sourcepub const fn checked_add(self, rhs: u64) -> Option<Self>
pub const fn checked_add(self, rhs: u64) -> Option<Self>
Return self + rhs returning None on overflow or if result exceeds MAX_POSITION.
Sourcepub const fn checked_sub(self, rhs: u64) -> Option<Self>
pub const fn checked_sub(self, rhs: u64) -> Option<Self>
Return self - rhs returning None on underflow.
Sourcepub const fn saturating_add(self, rhs: u64) -> Self
pub const fn saturating_add(self, rhs: u64) -> Self
Return self + rhs saturating at MAX_POSITION.
Sourcepub const fn saturating_sub(self, rhs: u64) -> Self
pub const fn saturating_sub(self, rhs: u64) -> Self
Return self - rhs saturating at zero.
Sourcepub const fn is_mmr_size(self) -> bool
pub const fn is_mmr_size(self) -> bool
Returns whether this is a valid MMR size.
The implementation verifies that (1) the size won’t result in overflow and (2) peaks in the MMR of the given size have strictly decreasing height, which is a necessary condition for MMR validity.