ValueEncoding

Trait ValueEncoding 

pub trait ValueEncoding:
    Clone
    + Send
    + Sync {
    type Value: CodecShared + Clone;
}
A wrapper around a value to indicate whether it is fixed or variable size. Having separate wrappers for fixed and variable size values allows us to use the same operation type for both fixed and variable size values, while still being able to parameterize the operation encoding by the value type.

Required Associated Types

type Value: CodecShared + Clone

The wrapped value type.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors