Module db

Module db 

Shared implementation for ordered Current QMDB variants.

This module contains impl blocks that are generic over ValueEncoding, allowing them to be used by both fixed and variable ordered QMDB implementations.

Structs

KeyValueProof
Proof information for verifying a key has a particular value in the database.

Type Aliases

Db
The generic Db type for ordered Current QMDB variants.