pub struct KeyValueProof<F: Graftable, K: Key, D: Digest, const N: usize> {
pub proof: OperationProof<F, D, N>,
pub next_key: K,
}Expand description
Proof information for verifying a key has a particular value in the database.
Fields
proof: OperationProof<F, D, N>next_key: KTrait Implementations
Sourceimpl<F: Clone + Graftable, K: Clone + Key, D: Clone + Digest, const N: usize> Clone for KeyValueProof<F, K, D, N>
impl<F: Clone + Graftable, K: Clone + Key, D: Clone + Digest, const N: usize> Clone for KeyValueProof<F, K, D, N>
Sourcefn clone(&self) -> KeyValueProof<F, K, D, N>
fn clone(&self) -> KeyValueProof<F, K, D, N>
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<F: Debug + Graftable, K: Debug + Key, D: Debug + Digest, const N: usize> Debug for KeyValueProof<F, K, D, N>
impl<F: Debug + Graftable, K: Debug + Key, D: Debug + Digest, const N: usize> Debug for KeyValueProof<F, K, D, N>
Sourceimpl<F: Graftable, K: Key, D: Digest, const N: usize> EncodeSize for KeyValueProof<F, K, D, N>
impl<F: Graftable, K: Key, D: Digest, const N: usize> EncodeSize for KeyValueProof<F, K, D, N>
Sourcefn encode_size(&self) -> usize
fn encode_size(&self) -> usize
Returns the encoded size of this value (in bytes).
Sourcefn encode_inline_size(&self) -> usize
fn encode_inline_size(&self) -> usize
Returns the encoded size excluding bytes passed to
BufsMut::push
during Write::write_bufs. Used to size the working buffer for inline
writes. Override alongside Write::write_bufs for types where large
Bytes fields go via push; failing to do so will over-allocate.Sourceimpl<F: PartialEq + Graftable, K: PartialEq + Key, D: PartialEq + Digest, const N: usize> PartialEq for KeyValueProof<F, K, D, N>
impl<F: PartialEq + Graftable, K: PartialEq + Key, D: PartialEq + Digest, const N: usize> PartialEq for KeyValueProof<F, K, D, N>
Sourcefn eq(&self, other: &KeyValueProof<F, K, D, N>) -> bool
fn eq(&self, other: &KeyValueProof<F, K, D, N>) -> bool
Tests for
self and other values to be equal, and is used by ==.Sourceimpl<F: Graftable, K: Key, D: Digest, const N: usize> Read for KeyValueProof<F, K, D, N>
impl<F: Graftable, K: Key, D: Digest, const N: usize> Read for KeyValueProof<F, K, D, N>
Sourceimpl<F: Graftable, K: Key, D: Digest, const N: usize> Write for KeyValueProof<F, K, D, N>
impl<F: Graftable, K: Key, D: Digest, const N: usize> Write for KeyValueProof<F, K, D, N>
Sourcefn write_bufs(&self, buf: &mut impl BufsMut)
fn write_bufs(&self, buf: &mut impl BufsMut)
Writes to a
BufsMut, allowing existing Bytes chunks to be
appended via BufsMut::push instead of written inline. Must encode
to the same format as Write::write. Defaults to Write::write.impl<F: Eq + Graftable, K: Eq + Key, D: Eq + Digest, const N: usize> Eq for KeyValueProof<F, K, D, N>
impl<F: Graftable, K: Key, D: Digest, const N: usize> StructuralPartialEq for KeyValueProof<F, K, D, N>