pub struct OperationProof<F: Family, D: Digest, const N: usize> {
pub loc: Location<F>,
pub chunk: [u8; N],
pub range_proof: RangeProof<F, D>,
}Expand description
A proof that a specific operation is currently active in the database.
Fields
loc: Location<F>The location of the operation in the db.
chunk: [u8; N]The status bitmap chunk that contains the bit corresponding the operation’s location.
range_proof: RangeProof<F, D>The range proof that incorporates activity status for the operation designated by loc.
Implementations
Sourceimpl<F: Graftable, D: Digest, const N: usize> OperationProof<F, D, N>
impl<F: Graftable, D: Digest, const N: usize> OperationProof<F, D, N>
Sourcepub async fn new<H: CHasher<Digest = D>, S: Storage<F, Digest = D>>(
hasher: &mut H,
status: &impl BitmapReadable<N>,
storage: &S,
loc: Location<F>,
ops_root: D,
) -> Result<Self, Error<F>>
pub async fn new<H: CHasher<Digest = D>, S: Storage<F, Digest = D>>( hasher: &mut H, status: &impl BitmapReadable<N>, storage: &S, loc: Location<F>, ops_root: D, ) -> Result<Self, Error<F>>
Return an inclusion proof that incorporates activity status for the operation designated by
loc.
Errors
Returns Error::OperationPruned if loc falls in a pruned bitmap chunk.