pub struct OpsRootWitness<F: Graftable, D: Digest> {
pub grafted_root: D,
pub pending_chunk_digest: F::PendingChunk<D>,
pub partial_chunk: Option<(u64, D)>,
}Expand description
Witness that a particular ops_root is committed by a current canonical root.
See the Canonical root structure section in the module documentation for the full layout.
Fields
grafted_root: DThe grafted-tree root committed by the canonical root.
pending_chunk_digest: F::PendingChunk<D>The pending-chunk contribution, if any.
partial_chunk: Option<(u64, D)>The trailing partial chunk contribution, if the bitmap length is not chunk-aligned:
(next_bit, partial_chunk_digest).
Implementations
Sourceimpl<F: Graftable, D: Digest> OpsRootWitness<F, D>
impl<F: Graftable, D: Digest> OpsRootWitness<F, D>
Sourcepub fn root<H: CHasher<Digest = D>>(
&self,
hasher: &StandardHasher<H>,
ops_root: &D,
) -> D
pub fn root<H: CHasher<Digest = D>>( &self, hasher: &StandardHasher<H>, ops_root: &D, ) -> D
Compute the canonical current root that commits to ops_root through this witness.
See the Canonical root structure section in the module documentation for the full layout.