pub struct RangeProof<F: Graftable, D: Digest> {
pub proof: Proof<F, D>,
pub pending_chunk_digest: F::PendingChunk<D>,
pub partial_chunk_digest: Option<D>,
pub ops_root: D,
}Expand description
A proof that a range of operations exist in the database.
Fields
proof: Proof<F, D>The Merkle digest material required to verify the proof.
pending_chunk_digest: F::PendingChunk<D>The pending-chunk contribution, if any.
partial_chunk_digest: Option<D>Digest of the bitmap’s trailing partial chunk, if any.
ops_root: DThe ops-tree root digest.
Implementations
Sourceimpl<F: Graftable, D: Digest> RangeProof<F, D>
impl<F: Graftable, D: Digest> RangeProof<F, D>
Sourcepub async fn new<H: CHasher<Digest = D>, S: Storage<F, Digest = D>, const N: usize>(
hasher: &StandardHasher<H>,
status: &impl BitmapReadable<N>,
storage: &S,
inactivity_floor: Location<F>,
range: Range<Location<F>>,
ops_root: D,
) -> Result<Self, Error<F>>
pub async fn new<H: CHasher<Digest = D>, S: Storage<F, Digest = D>, const N: usize>( hasher: &StandardHasher<H>, status: &impl BitmapReadable<N>, storage: &S, inactivity_floor: Location<F>, range: Range<Location<F>>, ops_root: D, ) -> Result<Self, Error<F>>
Create a new range proof for the provided range of operations.
Sourcepub async fn new_with_ops<H: CHasher<Digest = D>, C: Contiguous, S: Storage<F, Digest = D>, const N: usize>(
hasher: &StandardHasher<H>,
status: &impl BitmapReadable<N>,
storage: &S,
log: &C,
request: RangeProofSpec<F, D>,
) -> Result<(Self, Vec<C::Item>, Vec<[u8; N]>), Error<F>>
pub async fn new_with_ops<H: CHasher<Digest = D>, C: Contiguous, S: Storage<F, Digest = D>, const N: usize>( hasher: &StandardHasher<H>, status: &impl BitmapReadable<N>, storage: &S, log: &C, request: RangeProofSpec<F, D>, ) -> Result<(Self, Vec<C::Item>, Vec<[u8; N]>), Error<F>>
Returns a proof that the specified range of operations are part of the database, along with the operations from the range and their activity status chunks. A truncated range (from hitting the max) can be detected by looking at the length of the returned operations vector.
Errors
Returns Error::OperationPruned if start_loc falls in a pruned bitmap chunk.
Returns merkle::Error::LocationOverflow if start_loc > merkle::Family::MAX_LEAVES.
Returns merkle::Error::RangeOutOfBounds if start_loc >= number of leaves in the tree.
Sourcepub fn verify<H: CHasher<Digest = D>, O: Codec, const N: usize>(
&self,
root_hasher: &StandardHasher<H>,
start_loc: Location<F>,
ops: &[O],
chunks: &[[u8; N]],
root: &H::Digest,
) -> bool
pub fn verify<H: CHasher<Digest = D>, O: Codec, const N: usize>( &self, root_hasher: &StandardHasher<H>, start_loc: Location<F>, ops: &[O], chunks: &[[u8; N]], root: &H::Digest, ) -> bool
Return true if the given sequence of ops were applied starting at location start_loc in
the db with the provided root, and having the activity status described by chunks.