pub struct MerkleizedBatch<F: Family, D: Digest, K: Key, V: ValueEncoding> { /* private fields */ }Expand description
A speculative batch of operations whose root digest has been computed,
in contrast to UnmerkleizedBatch.
Implementations
Sourceimpl<F: Family, D: Digest, K: Key, V: ValueEncoding> MerkleizedBatch<F, D, K, V>where
Operation<K, V>: EncodeShared,
impl<F: Family, D: Digest, K: Key, V: ValueEncoding> MerkleizedBatch<F, D, K, V>where
Operation<K, V>: EncodeShared,
Sourcepub async fn get<E, C, H, T>(
&self,
key: &K,
db: &Immutable<F, E, K, V, C, H, T>,
) -> Result<Option<V::Value>, Error<F>>where
E: Context,
C: Mutable<Item = Operation<K, V>> + Persistable<Error = JournalError>,
C::Item: EncodeShared,
H: CHasher<Digest = D>,
T: Translator,
pub async fn get<E, C, H, T>(
&self,
key: &K,
db: &Immutable<F, E, K, V, C, H, T>,
) -> Result<Option<V::Value>, Error<F>>where
E: Context,
C: Mutable<Item = Operation<K, V>> + Persistable<Error = JournalError>,
C::Item: EncodeShared,
H: CHasher<Digest = D>,
T: Translator,
Read through: local diff -> ancestor diffs -> committed DB.
Sourcepub fn new_batch<H>(self: &Arc<Self>) -> UnmerkleizedBatch<F, H, K, V>where
H: CHasher<Digest = D>,
pub fn new_batch<H>(self: &Arc<Self>) -> UnmerkleizedBatch<F, H, K, V>where
H: CHasher<Digest = D>,
Create a new speculative batch of operations with this batch as its parent.
All uncommitted ancestors in the chain must be kept alive until the child (or any
descendant) is merkleized. Dropping an uncommitted ancestor causes data
loss detected at apply_batch time.