pub struct CompactUnmerkleizedBatch<F, H, K, V, S: Strategy>{ /* private fields */ }Expand description
A speculative batch for a compact immutable db.
Implementations
Sourceimpl<F, H, K, V, S> UnmerkleizedBatch<F, H, K, V, S>where
F: Family,
K: Key,
V: ValueEncoding,
H: Hasher,
S: Strategy,
Operation<F, K, V>: EncodeShared,
impl<F, H, K, V, S> UnmerkleizedBatch<F, H, K, V, S>where
F: Family,
K: Key,
V: ValueEncoding,
H: Hasher,
S: Strategy,
Operation<F, K, V>: EncodeShared,
pub fn set(self, key: K, value: V::Value) -> Self
Sourcepub fn merkleize<E, C>(
self,
db: &Db<F, E, K, V, H, C, S>,
metadata: Option<V::Value>,
inactivity_floor: Location<F>,
) -> Arc<MerkleizedBatch<F, H::Digest, K, V, S>> ⓘ
pub fn merkleize<E, C>( self, db: &Db<F, E, K, V, H, C, S>, metadata: Option<V::Value>, inactivity_floor: Location<F>, ) -> Arc<MerkleizedBatch<F, H::Digest, K, V, S>> ⓘ
Resolve mutations into operations, merkleize, and return an Arc<MerkleizedBatch>.
inactivity_floor is threaded through the commit operation for wire-format parity with
crate::qmdb::immutable::Immutable. It must be >= the database’s current floor
(monotonically non-decreasing) and at most the batch’s commit location
(total_size - 1); these bounds are validated, but the floor does not drive any local
pruning or retention in this variant.