pub struct CompactMerkleizedBatch<F: Family, D: Digest, V: ValueEncoding, S: Strategy>where
Operation<F, V>: EncodeShared,{ /* private fields */ }Expand description
A speculative batch whose root digest has been computed.
Implementations
Sourceimpl<F: Family, D: Digest, V: ValueEncoding, S: Strategy> MerkleizedBatch<F, D, V, S>where
Operation<F, V>: EncodeShared,
impl<F: Family, D: Digest, V: ValueEncoding, S: Strategy> MerkleizedBatch<F, D, V, S>where
Operation<F, V>: EncodeShared,
Sourcepub fn new_batch<H>(self: &Arc<Self>) -> UnmerkleizedBatch<F, H, V, S>where
H: Hasher<Digest = D>,
pub fn new_batch<H>(self: &Arc<Self>) -> UnmerkleizedBatch<F, H, V, S>where
H: Hasher<Digest = D>,
Create a new speculative batch with this one as its parent.