pub struct Batch<'a, E, K, V, T>{ /* private fields */ }Expand description
A mutable batch of writes and deletes staged against the current store state.
Implementations
Sourceimpl<'a, E, K, V, T> Batch<'a, E, K, V, T>
impl<'a, E, K, V, T> Batch<'a, E, K, V, T>
Sourcepub fn finalize(self, metadata: Option<V>) -> Changeset<K, V>
pub fn finalize(self, metadata: Option<V>) -> Changeset<K, V>
Finalize the batch into a changeset that can be applied to the store.