pub enum EngineError<F: Family, D: Digest> {
RootMismatch {
expected: D,
actual: D,
},
InvalidResponse,
InvalidTarget {
lower_bound_pos: Location<F>,
upper_bound_pos: Location<F>,
},
InvalidState,
SyncTargetRootUnchanged,
SyncStalled,
FinishChannelClosed,
}Variants
RootMismatch
Hash mismatch after sync
InvalidResponse
The source served a response that cannot satisfy the target and is not accepting feedback. Retrying cannot yield a different answer.
InvalidTarget
Invalid target parameters
InvalidState
Invalid client state
SyncTargetRootUnchanged
Sync target root unchanged
SyncStalled
Sync stalled - no pending fetches
FinishChannelClosed
Sync finish signal channel closed before finish was requested.