pub enum Request<F: Family> {
Operations {
size: Location<F>,
start: Location<F>,
max_ops: NonZeroU64,
},
Boundary {
size: Location<F>,
start: Location<F>,
},
}Expand description
A request for operations from a source’s log.
Variants
Operations
Fetch the operations in [start, start + max_ops).
Fields

max_ops: NonZeroU64Maximum number of operations to return.
Boundary
Fetch the single operation at start plus the pinned nodes at start, the lowest
location the client will retain. The proof in the response authenticates the pinned nodes,
so there is no way to request them on their own.
Implementations