pub struct Metrics { /* private fields */ }Expand description
Progress gauges updated by a sync flow.
Progress is expressed as the number of leaves synced against the total leaves in the current sync target. The gauges match once the flow has synced the latest target.
Implementations
Sourceimpl Metrics
impl Metrics
Sourcepub fn new(context: &impl Metrics) -> Self
pub fn new(context: &impl Metrics) -> Self
Register sync progress metrics on the provided context.
Sourcepub fn record_target(&self, leaf_count: u64)
pub fn record_target(&self, leaf_count: u64)
Record the leaf count of the current sync target.
Sourcepub fn record_synced(&self, leaf_count: u64)
pub fn record_synced(&self, leaf_count: u64)
Record the number of leaves synced so far.