pub struct Target<F: Family, D: Digest> {
pub root: D,
pub range: NonEmptyRange<Location<F>>,
}Expand description
Target state to sync to.
PartialEq, Eq, and Clone are implemented manually to avoid requiring F to implement
them.
Fields
root: DThe ops root the sync engine verifies streaming batches against.
range: NonEmptyRange<Location<F>>Range of operations to sync
Implementations