Shared queue with split writer/reader handles.
Provides concurrent access to a Queue with multiple writers and a single reader.
The reader can await new items using Reader::recv, which integrates
with select! for multiplexing with other futures.
Writers can be cloned to allow multiple tasks to enqueue items concurrently.
Structs
Functions
- init
- Initialize a shared queue and split into writer and reader handles.