Reader

Struct Reader 

pub struct Reader<E: Clock + Storage + Metrics, V: CodecShared> { /* private fields */ }
Reader handle for dequeuing and acknowledging items.

There should only be one reader per shared queue.

Implementations

Source

impl<E: Clock + Storage + Metrics, V: CodecShared> Reader<E, V>

pub async fn recv(&mut self) -> Result<Option<(u64, V)>, Error>

Receive the next unacknowledged item, waiting if necessary.

This method is designed for use with select!. It will:

  1. Return immediately if an unacked item is available
  2. Wait for the writer to enqueue new items if the queue is empty
  3. Return None if the writer is dropped (no more items will arrive)
Errors

Returns an error if the underlying storage operation fails.

pub async fn try_recv(&mut self) -> Result<Option<(u64, V)>, Error>

Try to dequeue the next unacknowledged item without waiting.

Returns None immediately if no unacked item is available.

Errors

Returns an error if the underlying storage operation fails.

pub async fn ack(&self, position: u64) -> Result<(), Error>

See Queue::ack.

Errors

Returns super::Error::PositionOutOfRange if the position is invalid.

pub async fn ack_up_to(&self, up_to: u64) -> Result<(), Error>

See Queue::ack_up_to.

Errors

Returns super::Error::PositionOutOfRange if up_to is invalid.

pub async fn ack_floor(&self) -> u64

pub async fn read_position(&self) -> u64

pub async fn is_empty(&self) -> bool

pub async fn reset(&self)