Sections

Trait Sections 

pub trait Sections {
    type Iter: Iterator<Item = u64>;

    // Required method
    fn sections(self) -> Self::Iter;
}
Section selector for storage operations that act on one or more sections.

Required Associated Types

type Iter: Iterator<Item = u64>

Iterator over selected sections.

Required Methods

fn sections(self) -> Self::Iter

Convert into selected section indices.

This trait does not impose ordering or uniqueness; each storage operation decides how to handle duplicates and missing sections.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl Sections for BTreeSet<u64>

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = IntoIter<u64>

Source

fn sections(self) -> Self::Iter

Source

impl Sections for Vec<u64>

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = IntoIter<u64>

Source

fn sections(self) -> Self::Iter

Source

impl Sections for u64

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = Once<u64>

Source

fn sections(self) -> Self::Iter

Source

impl<'a, const N: usize> Sections for &'a [u64; N]

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = Copied<Iter<'a, u64>>

Source

fn sections(self) -> Self::Iter

Source

impl<'a> Sections for &'a BTreeSet<u64>

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = Copied<Iter<'a, u64>>

Source

fn sections(self) -> Self::Iter

Source

impl<'a> Sections for &'a Vec<u64>

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = Copied<Iter<'a, u64>>

Source

fn sections(self) -> Self::Iter

Source

impl<'a> Sections for &'a [u64]

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = Copied<Iter<'a, u64>>

Source

fn sections(self) -> Self::Iter

Source

impl<const N: usize> Sections for [u64; N]

Available on crate feature std and neither commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_GAMMA nor commonware_stability_RESERVED only.
Source

type Iter = IntoIter<u64, N>

Source

fn sections(self) -> Self::Iter

Implementors