pub trait Factory: Unordered + Sized {
type Translator: Translator;
// Required method
fn new(ctx: impl Metrics, translator: Self::Translator) -> Self;
}Expand description
A trait for index types that can be constructed from a metrics context and translator.
Required Associated Types
Sourcetype Translator: Translator
type Translator: Translator
The translator used by this index.
Required Methods
Sourcefn new(ctx: impl Metrics, translator: Self::Translator) -> Self
fn new(ctx: impl Metrics, translator: Self::Translator) -> Self
Create a new index with the given metrics context and translator.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".