Authenticated journal implementation.
An authenticated journal maintains a contiguous journal of items alongside a Merkle-family structure. The item at index i in the journal corresponds to the leaf at Location i in the Merkle structure. This structure enables efficient proofs that an item is included in the journal at a specific location.
Structs
- Journal
- An append-only data structure that maintains a sequential journal of items alongside a Merkle-family structure. The item at index i in the journal corresponds to the leaf at Location i in the Merkle structure. This structure enables efficient proofs that an item is included in the journal at a specific location.
- Merkleized
Batch - A speculative batch whose root digest has been computed, in contrast to
UnmerkleizedBatch. - Unmerkleized
Batch - A speculative batch whose root digest has not yet been computed,
in contrast to
MerkleizedBatch.
Enums
- Error
- Errors that can occur when interacting with an authenticated journal.