Module authenticated

Module authenticated 

Authenticated journal implementation.

An authenticated journal maintains a contiguous journal of items alongside a Merkle Mountain Range (MMR). The item at index i in the journal corresponds to the leaf at Location i in the MMR. This structure enables efficient proofs that an item is included in the journal at a specific location.

Structs

Changeset
An owned changeset that can be applied to the journal.
Journal
An append-only data structure that maintains a sequential journal of items alongside a Merkle Mountain Range (MMR). The item at index i in the journal corresponds to the leaf at Location i in the MMR. This structure enables efficient proofs that an item is included in the journal at a specific location.
MerkleizedBatch
A speculative batch whose root digest has been computed, in contrast to UnmerkleizedBatch.
UnmerkleizedBatch
A speculative batch whose root digest has not yet been computed, in contrast to MerkleizedBatch.

Enums

Error
Errors that can occur when interacting with an authenticated journal.

Traits

BatchChain
A chain of batches whose items can be collected in append order.