pub struct Journal<E, C, H>{ /* private fields */ }Expand description
An append-only data structure that maintains a sequential journal of items alongside a Merkle Mountain Range (MMR). The item at index i in the journal corresponds to the leaf at Location i in the MMR. This structure enables efficient proofs that an item is included in the journal at a specific location.
Implementations
Sourceimpl<E, C, H> Journal<E, C, H>
impl<E, C, H> Journal<E, C, H>
Sourceimpl<E, C, H> Journal<E, C, H>where
E: Storage + Clock + Metrics,
C: Contiguous<Item: EncodeShared> + Persistable<Error = Error>,
H: Hasher,
impl<E, C, H> Journal<E, C, H>where
E: Storage + Clock + Metrics,
C: Contiguous<Item: EncodeShared> + Persistable<Error = Error>,
H: Hasher,
Sourceimpl<E, C, H> Journal<E, C, H>
impl<E, C, H> Journal<E, C, H>
Sourcepub async fn from_components(
mmr: Mmr<E, H::Digest>,
journal: C,
hasher: StandardHasher<H>,
apply_batch_size: u64,
) -> Result<Self, Error>
pub async fn from_components( mmr: Mmr<E, H::Digest>, journal: C, hasher: StandardHasher<H>, apply_batch_size: u64, ) -> Result<Self, Error>
Create a new Journal from the given components after aligning the MMR with the journal.
Sourcepub async fn append(&mut self, item: &C::Item) -> Result<Location, Error>
pub async fn append(&mut self, item: &C::Item) -> Result<Location, Error>
Append an item to the journal and update the MMR.
Sourcepub async fn apply_batch(
&mut self,
batch: Changeset<H::Digest, C::Item>,
) -> Result<(), Error>
pub async fn apply_batch( &mut self, batch: Changeset<H::Digest, C::Item>, ) -> Result<(), Error>
Apply a changeset to the journal.
A changeset is only valid if the journal has not been modified since the batch that produced it was created. Multiple batches can be forked from the same parent for speculative execution, but only one may be applied. Applying a stale changeset returns an error.
Sourceimpl<E, C, H> Journal<E, C, H>
impl<E, C, H> Journal<E, C, H>
Sourcepub async fn proof(
&self,
start_loc: Location,
max_ops: NonZeroU64,
) -> Result<(Proof<H::Digest>, Vec<C::Item>), Error>
pub async fn proof( &self, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<C::Item>), Error>
Generate a proof of inclusion for items starting at start_loc.
Returns a proof and the items corresponding to the leaves in the range start_loc..end_loc,
where end_loc is the minimum of the current item count and start_loc + max_ops.
Errors
- Returns Error::Mmr with MmrError::LocationOverflow if
start_loc> crate::mmr::MAX_LOCATION. - Returns Error::Mmr with MmrError::RangeOutOfBounds if
start_loc>= current item count. - Returns Error::Journal with crate::journal::Error::ItemPruned if
start_lochas been pruned.
Sourcepub async fn historical_proof(
&self,
historical_leaves: Location,
start_loc: Location,
max_ops: NonZeroU64,
) -> Result<(Proof<H::Digest>, Vec<C::Item>), Error>
pub async fn historical_proof( &self, historical_leaves: Location, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<C::Item>), Error>
Generate a historical proof with respect to the state of the MMR when it had
historical_leaves leaves.
Returns a proof and the items corresponding to the leaves in the range start_loc..end_loc,
where end_loc is the minimum of historical_leaves and start_loc + max_ops.
Errors
- Returns Error::Mmr with MmrError::RangeOutOfBounds if
start_loc>=historical_leavesorhistorical_leaves> number of items in the journal. - Returns Error::Journal with crate::journal::Error::ItemPruned if
start_lochas been pruned.
Sourceimpl<E, C, H> Journal<E, C, H>where
E: Storage + Clock + Metrics,
C: Contiguous<Item: EncodeShared> + Persistable<Error = Error>,
H: Hasher,
impl<E, C, H> Journal<E, C, H>where
E: Storage + Clock + Metrics,
C: Contiguous<Item: EncodeShared> + Persistable<Error = Error>,
H: Hasher,