pub struct MerkleizedBatch<F: Family, D: Digest, Item: Send + Sync, S: Strategy> { /* private fields */ }Expand description
A speculative batch whose root digest has been computed, in contrast to UnmerkleizedBatch.
Implementations
Sourceimpl<F: Family, D: Digest, Item: Send + Sync, S: Strategy> MerkleizedBatch<F, D, Item, S>
impl<F: Family, D: Digest, Item: Send + Sync, S: Strategy> MerkleizedBatch<F, D, Item, S>
Sourcepub fn root(
&self,
base: &Mem<F, D>,
hasher: &impl Hasher<F, Digest = D>,
inactive_peaks: usize,
) -> Result<D, Error<F>>
pub fn root( &self, base: &Mem<F, D>, hasher: &impl Hasher<F, Digest = D>, inactive_peaks: usize, ) -> Result<D, Error<F>>
Compute the root digest after this batch is applied using inactive_peaks and the bagging
carried by hasher.
This recomputes the root rather than reading a cache.
Sourcepub fn proof(
&self,
hasher: &impl Hasher<F, Digest = D>,
loc: Location<F>,
inactive_peaks: usize,
) -> Result<Proof<F, D>, Error<F>>
pub fn proof( &self, hasher: &impl Hasher<F, Digest = D>, loc: Location<F>, inactive_peaks: usize, ) -> Result<Proof<F, D>, Error<F>>
Inclusion proof for the element at loc.
Sourcepub fn range_proof(
&self,
hasher: &impl Hasher<F, Digest = D>,
range: Range<Location<F>>,
inactive_peaks: usize,
) -> Result<Proof<F, D>, Error<F>>
pub fn range_proof( &self, hasher: &impl Hasher<F, Digest = D>, range: Range<Location<F>>, inactive_peaks: usize, ) -> Result<Proof<F, D>, Error<F>>
Inclusion proof for all elements in range.
Sourcepub fn new_batch<H: Hasher<Digest = D>>(
self: &Arc<Self>,
) -> UnmerkleizedBatch<F, H, Item, S>where
Item: Encode,
pub fn new_batch<H: Hasher<Digest = D>>(
self: &Arc<Self>,
) -> UnmerkleizedBatch<F, H, Item, S>where
Item: Encode,
Create a new speculative batch of operations with this batch as its parent.
The batch becomes invalid if any ancestor is dropped before being applied, or a sibling fork has been applied.