MerkleizedBatch

Struct MerkleizedBatch 

pub struct MerkleizedBatch<F: Family, D: Digest, Item: Send + Sync> { /* private fields */ }
A speculative batch whose root digest has been computed, in contrast to UnmerkleizedBatch.

Implementations

Source

impl<F: Family, D: Digest, Item: Send + Sync> MerkleizedBatch<F, D, Item>

pub fn root(&self) -> D

Return the root digest of the authenticated journal after this batch is applied.

pub fn new_batch<H: Hasher<Digest = D>>( self: &Arc<Self>, ) -> UnmerkleizedBatch<F, H, Item>
where Item: Encode,

Create a new speculative batch of operations with this batch as its parent.

All uncommitted ancestors in the chain must be kept alive until the child (or any descendant) is merkleized. Dropping an uncommitted ancestor causes data loss detected at apply_batch time.