MerkleizedBatch

Struct MerkleizedBatch 

pub struct MerkleizedBatch<'a, H: Hasher, P: Readable<H::Digest>, Item> { /* private fields */ }
A speculative batch whose root digest has been computed, in contrast to UnmerkleizedBatch.

Implementations

Source

impl<'a, H: Hasher, P: Readable<H::Digest>, Item> MerkleizedBatch<'a, H, P, Item>

pub fn root(&self) -> H::Digest

Return the root digest of the authenticated journal after this batch is applied.

Source

impl<'a, H: Hasher, P: Readable<H::Digest>, Item: Send + Sync + Encode> MerkleizedBatch<'a, H, P, Item>

pub fn new_batch(&self) -> UnmerkleizedBatch<'_, H, Self, Item>

Create a new speculative batch of operations with this batch as its parent.

Source

impl<'a, H: Hasher, P, Item: Send + Sync> MerkleizedBatch<'a, H, P, Item>
where P: Readable<H::Digest> + BatchChainInfo<H::Digest> + BatchChain<Item>,

pub fn finalize(self) -> Changeset<H::Digest, Item>

Consume this batch, collecting the changes from its ancestors and itself into a Changeset which can be applied to the journal.