pub struct MerkleizedBatch<'a, H: Hasher, P: Readable<H::Digest>, Item> { /* private fields */ }Expand description
A speculative batch whose root digest has been computed, in contrast to UnmerkleizedBatch.
Implementations
Sourceimpl<'a, H: Hasher, P: Readable<H::Digest>, Item: Send + Sync + Encode> MerkleizedBatch<'a, H, P, Item>
impl<'a, H: Hasher, P: Readable<H::Digest>, Item: Send + Sync + Encode> MerkleizedBatch<'a, H, P, Item>
Sourcepub fn new_batch(&self) -> UnmerkleizedBatch<'_, H, Self, Item>
pub fn new_batch(&self) -> UnmerkleizedBatch<'_, H, Self, Item>
Create a new speculative batch of operations with this batch as its parent.