pub trait BatchChain<Item> {
// Required method
fn collect(&self, into: &mut Vec<Arc<Vec<Item>>>);
}Expand description
A chain of batches whose items can be collected in append order.
pub trait BatchChain<Item> {
// Required method
fn collect(&self, into: &mut Vec<Arc<Vec<Item>>>);
}A chain of batches whose items can be collected in append order.