Position-based journal for variable-length items.
The data blobs are the source of truth. The offsets journal provides indexed access and records the preferred recovery point for replaying data to rebuild offset entries.
Structs
- Config
- Configuration for a Journal.
- Journal
- A contiguous journal with variable-size entries.
- Prepared
Append - Items encoded for a deferred append, created by
Journal::prepare_appendand consumed byJournal::append_prepared. - Reader
- A reader over a variable journal.