Segmented journals with section-based storage.
This module provides journal implementations that organize data into sections, where each section is stored in a separate blob.
Segmented journals with section-based storage.
This module provides journal implementations that organize data into sections, where each section is stored in a separate blob.