pub struct ProofStore<F: Family, D> { /* private fields */ }Expand description
A store derived from a Proof that can be used to generate proofs over any sub-range of the original range.
Implementations
Sourceimpl<F: Family, D: Digest> ProofStore<F, D>
impl<F: Family, D: Digest> ProofStore<F, D>
Sourcepub fn new<H, E>(
hasher: &H,
proof: &Proof<F, D>,
elements: &[E],
start_loc: Location<F>,
root: &D,
) -> Result<Self, Error<F>>
pub fn new<H, E>( hasher: &H, proof: &Proof<F, D>, elements: &[E], start_loc: Location<F>, root: &D, ) -> Result<Self, Error<F>>
Create a ProofStore from a Proof of inclusion of the provided range of elements from
the structure with root root. The resulting store can be used to generate range proofs
over any sub-range of the original range. Returns an error if the proof is invalid or could
not be verified against the given root.
The fold prefix accumulator from the proof is stored internally so that sub-range proofs with different fold prefix boundaries can be generated without requiring individual peak digests.
Sourcepub fn range_proof<H: Hasher<F, Digest = D>>(
&self,
hasher: &H,
range: Range<Location<F>>,
) -> Result<Proof<F, D>, Error<F>>
pub fn range_proof<H: Hasher<F, Digest = D>>( &self, hasher: &H, range: Range<Location<F>>, ) -> Result<Proof<F, D>, Error<F>>
Return a range proof for the nodes corresponding to the given location range.
The sub-range’s fold prefix accumulator is derived from the stored fold accumulator (covering the original proof’s fold prefix peaks) plus any additional peaks that are individually available in the store (original range peaks now preceding the sub-range).
Sourcepub fn multi_proof(
&self,
locations: &[Location<F>],
peaks: &[(Position<F>, D)],
) -> Result<Proof<F, D>, Error<F>>
pub fn multi_proof( &self, locations: &[Location<F>], peaks: &[(Position<F>, D)], ) -> Result<Proof<F, D>, Error<F>>
Return a multi proof for the elements corresponding to the given locations.
Since multi-proofs require individual node digests (not fold accumulators), callers must supply any peak digests that fall in the fold prefix of the original proof. These are the peaks entirely before the original range’s start location. If the original range started at location 0, no peaks are needed.