Db

Struct Db 

pub struct Db<E: Storage + Clock + Metrics, C: Contiguous<Item: CodecShared>, I: UnorderedIndex<Value = Location>, H: Hasher, U: Send + Sync, const N: usize> { /* private fields */ }
A Current QMDB implementation generic over ordered/unordered keys and variable/fixed values.

Implementations

Source

impl<E, K, V, C, I, H, U, const N: usize> Db<E, C, I, H, U, N>
where E: Storage + Clock + Metrics, K: Key, V: ValueEncoding, U: Update<K, V>, C: Contiguous<Item = Operation<K, V, U>>, I: UnorderedIndex<Value = Location>, H: Hasher, Operation<K, V, U>: Codec,

pub const fn inactivity_floor_loc(&self) -> Location

Return the inactivity floor location. This is the location before which all operations are known to be inactive. Operations before this point can be safely pruned.

pub const fn is_empty(&self) -> bool

Whether the snapshot currently has no active keys.

pub async fn get_metadata(&self) -> Result<Option<V::Value>, Error>

Get the metadata associated with the last commit.

pub fn verify_range_proof( hasher: &mut H, proof: &RangeProof<H::Digest>, start_loc: Location, ops: &[Operation<K, V, U>], chunks: &[[u8; N]], root: &H::Digest, ) -> bool

Return true if the given sequence of ops were applied starting at location start_loc in the log with the provided root, having the activity status described by chunks.

Source

impl<E, K, V, U, C, I, H, const N: usize> Db<E, C, I, H, U, N>
where E: Storage + Clock + Metrics, K: Key, V: ValueEncoding, U: Update<K, V>, C: Contiguous<Item = Operation<K, V, U>>, I: UnorderedIndex<Value = Location>, H: Hasher, Operation<K, V, U>: Codec,

pub const fn root(&self) -> H::Digest

Returns the canonical root. See the Root structure section in the module documentation.

pub fn ops_root(&self) -> H::Digest

Returns the ops MMR root.

This is the root of the raw operations log, without the activity bitmap. It is used as the sync target because the sync engine verifies batches against the ops MMR, not the canonical root.

See the Root structure section in the module documentation.

pub fn new_batch( &self, ) -> UnmerkleizedBatch<'_, E, K, V, C, I, H, U, Mmr<E, H::Digest>, Mmr<H::Digest>, BitMap<N>, N>

Create a new speculative batch of operations with this database as its parent.

pub async fn range_proof( &self, hasher: &mut H, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(RangeProof<H::Digest>, Vec<Operation<K, V, U>>, Vec<[u8; N]>), Error>

Returns a proof that the specified range of operations are part of the database, along with the operations from the range. A truncated range (from hitting the max) can be detected by looking at the length of the returned operations vector. Also returns the bitmap chunks required to verify the proof.

Errors

Returns Error::OperationPruned if start_loc falls in a pruned bitmap chunk. Returns mmr::Error::LocationOverflow if start_loc > mmr::MAX_LOCATION. Returns mmr::Error::RangeOutOfBounds if start_loc >= number of leaves in the MMR.

Source

impl<E, K, V, U, C, I, H, const N: usize> Db<E, C, I, H, U, N>
where E: Storage + Clock + Metrics, K: Key, V: ValueEncoding, U: Update<K, V>, C: Mutable<Item = Operation<K, V, U>>, I: UnorderedIndex<Value = Location>, H: Hasher, Operation<K, V, U>: Codec,

pub async fn ops_historical_proof( &self, historical_size: Location, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<K, V, U>>), Error>

Returns an ops-level historical proof for the specified range.

Unlike range_proof which returns grafted proofs incorporating the activity bitmap, this returns standard MMR proofs suitable for state sync.

pub async fn prune(&mut self, prune_loc: Location) -> Result<(), Error>

Prunes historical operations prior to prune_loc. This does not affect the db’s root or snapshot.

Errors
Source

impl<E, K, V, U, C, I, H, const N: usize> Db<E, C, I, H, U, N>
where E: Storage + Clock + Metrics, K: Key, V: ValueEncoding, U: Update<K, V>, C: Mutable<Item = Operation<K, V, U>> + Persistable<Error = Error>, I: UnorderedIndex<Value = Location>, H: Hasher, Operation<K, V, U>: Codec,

pub async fn sync(&self) -> Result<(), Error>

Sync all database state to disk.

pub async fn destroy(self) -> Result<(), Error>

Destroy the db, removing all data from disk.

Source

impl<E, K, V, U, C, I, H, const N: usize> Db<E, C, I, H, U, N>
where E: Storage + Clock + Metrics, K: Key, V: ValueEncoding, U: Update<K, V> + 'static, C: Mutable<Item = Operation<K, V, U>> + Persistable<Error = Error>, I: UnorderedIndex<Value = Location>, H: Hasher, Operation<K, V, U>: Codec,

pub async fn apply_batch( &mut self, batch: Changeset<K, H::Digest, Operation<K, V, U>, N>, ) -> Result<Range<Location>, Error>

Apply a changeset to the database.

A changeset is only valid if the database has not been modified since the batch that produced it was created. Multiple batches can be forked from the same parent for speculative execution, but only one may be applied. Applying a stale changeset returns Error::StaleChangeset.

Returns the range of locations written.

Source

impl<E: Storage + Clock + Metrics, C: Contiguous<Item = Operation<K, V>>, K: Key, V: ValueEncoding, I: OrderedIndex<Value = Location>, H: Hasher, const N: usize> Db<E, C, I, H, Update<K, V>, N>
where Operation<K, V>: Codec, V::Value: Send + Sync,

pub async fn get(&self, key: &K) -> Result<Option<V::Value>, Error>

Get the value of key in the db, or None if it has no value.

pub fn verify_key_value_proof( hasher: &mut H, key: K, value: V::Value, proof: &KeyValueProof<K, H::Digest, N>, root: &H::Digest, ) -> bool

Return true if the proof authenticates that key currently has value value in the db with the provided root.

pub async fn get_span( &self, key: &K, ) -> Result<Option<(Location, Update<K, V>)>, Error>

Get the operation that currently defines the span whose range contains key, or None if the DB is empty.

pub async fn stream_range<'a>( &'a self, start: K, ) -> Result<impl Stream<Item = Result<(K, V::Value), Error>> + 'a, Error>
where V: 'a,

Streams all active (key, value) pairs in the database in key order, starting from the first active key greater than or equal to start.

pub fn verify_exclusion_proof( hasher: &mut H, key: &K, proof: &ExclusionProof<K, V, H::Digest, N>, root: &H::Digest, ) -> bool

Return true if the proof authenticates that key does not exist in the db with the provided root.

Source

impl<E: Storage + Clock + Metrics, C: Mutable<Item = Operation<K, V>>, K: Key, V: ValueEncoding, I: OrderedIndex<Value = Location>, H: Hasher, const N: usize> Db<E, C, I, H, Update<K, V>, N>
where Operation<K, V>: Codec, V::Value: Send + Sync,

pub async fn key_value_proof( &self, hasher: &mut H, key: K, ) -> Result<KeyValueProof<K, H::Digest, N>, Error>

Generate and return a proof of the current value of key, along with the other KeyValueProof required to verify the proof. Returns KeyNotFound error if the key is not currently assigned any value.

Errors

Returns Error::KeyNotFound if the key is not currently assigned any value.

pub async fn exclusion_proof( &self, hasher: &mut H, key: &K, ) -> Result<ExclusionProof<K, V, H::Digest, N>, Error>

Generate and return a proof that the specified key does not exist in the db.

Errors

Returns Error::KeyExists if the key exists in the db.

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: FixedValue, H: Hasher, T: Translator, const P: usize, const N: usize> Db<E, Journal<E, Operation<K, FixedEncoding<V>, Update<K, FixedEncoding<V>>>>, Index<T, Location, P>, H, Update<K, FixedEncoding<V>>, N>

pub async fn init(context: E, config: Config<T>) -> Result<Self, Error>

Initializes a Db authenticated database from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: FixedValue, H: Hasher, T: Translator, const N: usize> Db<E, Journal<E, Operation<K, FixedEncoding<V>, Update<K, FixedEncoding<V>>>>, Index<T, Location>, H, Update<K, FixedEncoding<V>>, N>

pub async fn init(context: E, config: Config<T>) -> Result<Self, Error>

Initializes a Db from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: RStorage + Clock + Metrics, K: Key, V: VariableValue, H: Hasher, T: Translator, const P: usize, const N: usize> Db<E, Journal<E, Operation<K, VariableEncoding<V>, Update<K, VariableEncoding<V>>>>, Index<T, Location, P>, H, Update<K, VariableEncoding<V>>, N>
where Operation<K, V>: Codec,

pub async fn init( context: E, config: Config<T, <Operation<K, V> as Read>::Cfg>, ) -> Result<Self, Error>

Initializes a Db from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: RStorage + Clock + Metrics, K: Key, V: VariableValue, H: Hasher, T: Translator, const N: usize> Db<E, Journal<E, Operation<K, VariableEncoding<V>, Update<K, VariableEncoding<V>>>>, Index<T, Location>, H, Update<K, VariableEncoding<V>>, N>
where Operation<K, V>: Codec,

pub async fn init( context: E, config: Config<T, <Operation<K, V> as Read>::Cfg>, ) -> Result<Self, Error>

Initializes a Db from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: Storage + Clock + Metrics, C: Contiguous<Item = Operation<K, V>>, K: Array, V: ValueEncoding, I: UnorderedIndex<Value = Location>, H: Hasher, const N: usize> Db<E, C, I, H, Update<K, V>, N>
where Operation<K, V>: Codec, V::Value: Send + Sync,

pub async fn get(&self, key: &K) -> Result<Option<V::Value>, Error>

Get the value of key in the db, or None if it has no value.

pub fn verify_key_value_proof( hasher: &mut H, key: K, value: V::Value, proof: &KeyValueProof<H::Digest, N>, root: &H::Digest, ) -> bool

Return true if the proof authenticates that key currently has value value in the db with the provided root.

Source

impl<E: Storage + Clock + Metrics, C: Mutable<Item = Operation<K, V>>, K: Array, V: ValueEncoding, I: UnorderedIndex<Value = Location>, H: Hasher, const N: usize> Db<E, C, I, H, Update<K, V>, N>
where Operation<K, V>: Codec, V::Value: Send + Sync,

pub async fn key_value_proof( &self, hasher: &mut H, key: K, ) -> Result<KeyValueProof<H::Digest, N>, Error>

Generate and return a proof of the current value of key, along with the other KeyValueProof required to verify the proof. Returns KeyNotFound error if the key is not currently assigned any value.

Errors

Returns Error::KeyNotFound if the key is not currently assigned any value.

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: FixedValue, H: Hasher, T: Translator, const P: usize, const N: usize> Db<E, Journal<E, Operation<K, FixedEncoding<V>, Update<K, FixedEncoding<V>>>>, Index<T, Location, P>, H, Update<K, FixedEncoding<V>>, N>

pub async fn init(context: E, config: Config<T>) -> Result<Self, Error>

Initializes a Db authenticated database from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: FixedValue, H: Hasher, T: Translator, const N: usize> Db<E, Journal<E, Operation<K, FixedEncoding<V>, Update<K, FixedEncoding<V>>>>, Index<T, Location>, H, Update<K, FixedEncoding<V>>, N>

pub async fn init(context: E, config: Config<T>) -> Result<Self, Error>

Initializes a Db authenticated database from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: VariableValue, H: Hasher, T: Translator, const P: usize, const N: usize> Db<E, Journal<E, Operation<K, VariableEncoding<V>, Update<K, VariableEncoding<V>>>>, Index<T, Location, P>, H, Update<K, VariableEncoding<V>>, N>
where Operation<K, V>: Read,

pub async fn init( context: E, config: Config<T, <Operation<K, V> as Read>::Cfg>, ) -> Result<Self, Error>

Initializes a Db from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: VariableValue, H: Hasher, T: Translator, const N: usize> Db<E, Journal<E, Operation<K, VariableEncoding<V>, Update<K, VariableEncoding<V>>>>, Index<T, Location>, H, Update<K, VariableEncoding<V>>, N>
where Operation<K, V>: Read,

pub async fn init( context: E, config: Config<T, <Operation<K, V> as Read>::Cfg>, ) -> Result<Self, Error>

Initializes a Db from the given config. Leverages parallel Merkleization to initialize the bitmap MMR if a thread pool is provided.