Ordered variants of a crate::qmdb::current authenticated database.
These variants maintain the lexicographic-next active key for each active key, enabling exclusion proofs via ExclusionProof. This adds overhead compared to super::unordered variants.
Variants:
Modules
- db
- Shared implementation for ordered Current QMDB variants.
- fixed
- An ordered variant of a crate::qmdb::current authenticated database optimized for fixed-size values.
- variable
- An ordered variant of a crate::qmdb::current authenticated database for variable-size values
Enums
- Exclusion
Proof - Proof that a key has no assigned value in the database.