pub enum ExclusionProof<F: Graftable, K: Key, V: ValueEncoding, D: Digest, const N: usize> {
KeyValue(OperationProof<F, D, N>, Update<K, V>),
Commit(OperationProof<F, D, N>, Option<V::Value>),
}Expand description
Proof that a key has no assigned value in the database.
When the database has active keys, exclusion is proven by showing the key falls within a span between two adjacent active keys. Otherwise exclusion is proven by showing the database contains no active keys through the most recent commit operation.
Verify using Db::verify_exclusion_proof.
Variants
KeyValue(OperationProof<F, D, N>, Update<K, V>)
Proves that two keys are active in the database and adjacent to each other in the key ordering. Any key falling between them (non-inclusively) can be proven excluded.
Commit(OperationProof<F, D, N>, Option<V::Value>)
Proves that the database has no active keys, allowing any key to be proven excluded. Specifically, the proof establishes the most recent Commit operation has an activity floor equal to its own location, which is a necessary and sufficient condition for an empty database.