pub enum Error<F: Family> {
DataCorrupted(&'static str),
Merkle(Error<F>),
Metadata(Error),
Journal(Error),
Runtime(Error),
OperationPruned(Location<F>),
KeyNotFound,
KeyExists,
UnexpectedData(Location<F>),
LocationOutOfBounds(Location<F>, Location<F>),
PruneBeyondMinRequired(Location<F>, Location<F>),
StaleBatch {
db_size: u64,
batch_db_size: u64,
batch_base_size: u64,
},
}Expand description
Errors that can occur when interacting with an authenticated database.
Variants
DataCorrupted(&'static str)
Merkle(Error<F>)
Metadata(Error)
Journal(Error)
Runtime(Error)
OperationPruned(Location<F>)
KeyNotFound
The requested key was not found in the snapshot.
KeyExists
The key exists in the db, so we cannot prove its exclusion.
UnexpectedData(Location<F>)
LocationOutOfBounds(Location<F>, Location<F>)
PruneBeyondMinRequired(Location<F>, Location<F>)
StaleBatch
The batch was created from a different database state than the current one.