pub struct Immutable<F: Family, E: Context, K: Key, V: ValueEncoding, C: Mutable<Item = Operation<K, V>> + Persistable<Error = Error>, H: CHasher, T: Translator>where
C::Item: EncodeShared,{ /* private fields */ }Expand description
An authenticated database that only supports adding new keyed values (no updates or deletions).
Invariant
A key must be set at most once across the database history. Writing the same key more than once is undefined behavior.
Use fixed::Db or variable::Db for concrete instantiations.
Implementations
Sourceimpl<F, E, K, V, C, H, T> Immutable<F, E, K, V, C, H, T>where
F: Family,
E: Context,
K: Key,
V: ValueEncoding,
C: Mutable<Item = Operation<K, V>> + Persistable<Error = Error>,
C::Item: EncodeShared,
H: CHasher,
T: Translator,
impl<F, E, K, V, C, H, T> Immutable<F, E, K, V, C, H, T>where
F: Family,
E: Context,
K: Key,
V: ValueEncoding,
C: Mutable<Item = Operation<K, V>> + Persistable<Error = Error>,
C::Item: EncodeShared,
H: CHasher,
T: Translator,
Sourcepub fn to_batch(&self) -> Arc<MerkleizedBatch<F, H::Digest, K, V>>
pub fn to_batch(&self) -> Arc<MerkleizedBatch<F, H::Digest, K, V>>
Create an initial MerkleizedBatch from the committed DB state.
Sourceimpl<F: Family, E: Storage + Clock + Metrics, K: Array, V: FixedValue, H: Hasher, T: Translator> Immutable<F, E, K, FixedEncoding<V>, Journal<E, Operation<K, FixedEncoding<V>>>, H, T>
impl<F: Family, E: Storage + Clock + Metrics, K: Array, V: FixedValue, H: Hasher, T: Translator> Immutable<F, E, K, FixedEncoding<V>, Journal<E, Operation<K, FixedEncoding<V>>>, H, T>
Sourceimpl<F: Family, E: Storage + Clock + Metrics, K: Key, V: VariableValue, H: Hasher, T: Translator> Immutable<F, E, K, VariableEncoding<V>, Journal<E, Operation<K, VariableEncoding<V>>>, H, T>
impl<F: Family, E: Storage + Clock + Metrics, K: Key, V: VariableValue, H: Hasher, T: Translator> Immutable<F, E, K, VariableEncoding<V>, Journal<E, Operation<K, VariableEncoding<V>>>, H, T>
Sourceimpl<F, E, K, V, C, H, T> Immutable<F, E, K, V, C, H, T>where
F: Family,
E: Context,
K: Key,
V: ValueEncoding,
C: Mutable<Item = Operation<K, V>> + Persistable<Error = Error>,
C::Item: EncodeShared,
H: CHasher,
T: Translator,
impl<F, E, K, V, C, H, T> Immutable<F, E, K, V, C, H, T>where
F: Family,
E: Context,
K: Key,
V: ValueEncoding,
C: Mutable<Item = Operation<K, V>> + Persistable<Error = Error>,
C::Item: EncodeShared,
H: CHasher,
T: Translator,
Sourcepub async fn size(&self) -> Location<F>
pub async fn size(&self) -> Location<F>
Return the Location of the next operation appended to this db.
Sourcepub async fn bounds(&self) -> Range<Location<F>>
pub async fn bounds(&self) -> Range<Location<F>>
Return [start, end) where start and end - 1 are the Locations of the oldest and newest
retained operations respectively.
Sourcepub async fn get(&self, key: &K) -> Result<Option<V::Value>, Error<F>>
pub async fn get(&self, key: &K) -> Result<Option<V::Value>, Error<F>>
Get the value of key in the db, or None if it has no value or its corresponding operation
has been pruned.
Sourcepub async fn get_metadata(&self) -> Result<Option<V::Value>, Error<F>>
pub async fn get_metadata(&self) -> Result<Option<V::Value>, Error<F>>
Get the metadata associated with the last commit.
Sourcepub async fn historical_proof(
&self,
op_count: Location<F>,
start_loc: Location<F>,
max_ops: NonZeroU64,
) -> Result<(Proof<F, H::Digest>, Vec<Operation<K, V>>), Error<F>>
pub async fn historical_proof( &self, op_count: Location<F>, start_loc: Location<F>, max_ops: NonZeroU64, ) -> Result<(Proof<F, H::Digest>, Vec<Operation<K, V>>), Error<F>>
Analogous to proof but with respect to the state of the database when it had op_count
operations.
Errors
Returns crate::merkle::Error::LocationOverflow if op_count or start_loc >
crate::merkle::Family::MAX_LEAVES.
Returns crate::merkle::Error::RangeOutOfBounds if op_count > number of operations, or
if start_loc >= op_count.
Returns Error::OperationPruned if start_loc has been pruned.
Sourcepub async fn proof(
&self,
start_index: Location<F>,
max_ops: NonZeroU64,
) -> Result<(Proof<F, H::Digest>, Vec<Operation<K, V>>), Error<F>>
pub async fn proof( &self, start_index: Location<F>, max_ops: NonZeroU64, ) -> Result<(Proof<F, H::Digest>, Vec<Operation<K, V>>), Error<F>>
Generate and return:
- a proof of all operations applied to the db in the range starting at (and including)
location
start_loc, and ending at the first of either:- the last operation performed, or
- the operation
max_opsfrom the start.
- the operations corresponding to the leaves in this range.
Sourcepub async fn prune(&mut self, loc: Location<F>) -> Result<(), Error<F>>
pub async fn prune(&mut self, loc: Location<F>) -> Result<(), Error<F>>
Prune operations prior to prune_loc. This does not affect the db’s root, but it will
affect retrieval of any keys that were set prior to prune_loc.
Errors
- Returns Error::PruneBeyondMinRequired if
prune_loc> last commit location. - Returns crate::merkle::Error::LocationOverflow if
prune_loc> crate::merkle::Family::MAX_LEAVES.
Sourcepub async fn rewind(&mut self, size: Location<F>) -> Result<(), Error<F>>
pub async fn rewind(&mut self, size: Location<F>) -> Result<(), Error<F>>
Rewind the database to size operations, where size is the location of the next append.
This rewinds both the operations journal and its Merkle structure to the historical
state at size, and removes rewound set operations from the in-memory snapshot.
Errors
Returns an error when:
sizeis not a valid rewind target- the target’s required logical range is not fully retained (for immutable, this means the oldest retained location is already beyond the rewind boundary)
size - 1is not a commit operation
Any error from this method is fatal for this handle. Rewind may mutate journal state
before this method finishes rebuilding in-memory rewind state. Callers must drop this
database handle after any Err from rewind and reopen from storage.
A successful rewind is not restart-stable until a subsequent Immutable::commit or
Immutable::sync.
Sourcepub async fn pinned_nodes_at(
&self,
loc: Location<F>,
) -> Result<Vec<H::Digest>, Error<F>>
pub async fn pinned_nodes_at( &self, loc: Location<F>, ) -> Result<Vec<H::Digest>, Error<F>>
Return the pinned Merkle nodes at the given location.
Sourcepub async fn sync(&self) -> Result<(), Error<F>>
pub async fn sync(&self) -> Result<(), Error<F>>
Sync all database state to disk. While this isn’t necessary to ensure durability of committed operations, periodic invocation may reduce memory usage and the time required to recover the database on restart.
Sourcepub async fn commit(&self) -> Result<(), Error<F>>
pub async fn commit(&self) -> Result<(), Error<F>>
Durably commit the journal state published by prior Immutable::apply_batch calls.
Sourcepub async fn destroy(self) -> Result<(), Error<F>>
pub async fn destroy(self) -> Result<(), Error<F>>
Destroy the db, removing all data from disk.
Sourcepub fn new_batch(&self) -> UnmerkleizedBatch<F, H, K, V>
pub fn new_batch(&self) -> UnmerkleizedBatch<F, H, K, V>
Create a new speculative batch of operations with this database as its parent.
Sourcepub async fn apply_batch(
&mut self,
batch: Arc<MerkleizedBatch<F, H::Digest, K, V>>,
) -> Result<Range<Location<F>>, Error<F>>
pub async fn apply_batch( &mut self, batch: Arc<MerkleizedBatch<F, H::Digest, K, V>>, ) -> Result<Range<Location<F>>, Error<F>>
Apply a batch::MerkleizedBatch to the database.
A batch is valid only if every batch applied to the database since this batch’s
ancestor chain was created is an ancestor of this batch. Applying a batch from a
different fork returns Error::StaleBatch.
Returns the range of locations written.
This publishes the batch to the in-memory database state and appends it to the
journal, but does not durably commit it. Call Immutable::commit or
Immutable::sync to guarantee durability.