Immutable

Struct Immutable 

pub struct Immutable<E: RStorage + Clock + Metrics, K: Array, V: VariableValue, H: CHasher, T: Translator> { /* private fields */ }
An authenticated database that only supports adding new keyed values (no updates or deletions), where values can have varying sizes.

Implementations

Source

impl<E: RStorage + Clock + Metrics, K: Array, V: VariableValue, H: CHasher, T: Translator> Immutable<E, K, V, H, T>

pub async fn size(&self) -> Location

Return the Location of the next operation appended to this db.

pub async fn bounds(&self) -> Range<Location>

Return [start, end) where start and end - 1 are the Locations of the oldest and newest retained operations respectively.

pub async fn get(&self, key: &K) -> Result<Option<V>, Error>

Get the value of key in the db, or None if it has no value or its corresponding operation has been pruned.

pub async fn get_metadata(&self) -> Result<Option<V>, Error>

Get the metadata associated with the last commit.

pub async fn historical_proof( &self, op_count: Location, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<K, V>>), Error>

Analogous to proof but with respect to the state of the database when it had op_count operations.

Errors

Returns crate::mmr::Error::LocationOverflow if op_count or start_loc > crate::mmr::MAX_LOCATION. Returns crate::mmr::Error::RangeOutOfBounds if op_count > number of operations, or if start_loc >= op_count. Returns Error::OperationPruned if start_loc has been pruned.

pub async fn prune(&mut self, loc: Location) -> Result<(), Error>

Prune operations prior to prune_loc. This does not affect the db’s root, but it will affect retrieval of any keys that were set prior to prune_loc.

Errors
pub fn root(&self) -> H::Digest

Return the root of the db.

pub async fn proof( &self, start_index: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<K, V>>), Error>

Generate and return:

  1. a proof of all operations applied to the db in the range starting at (and including) location start_loc, and ending at the first of either:
    • the last operation performed, or
    • the operation max_ops from the start.
  2. the operations corresponding to the leaves in this range.
pub async fn init( context: E, cfg: Config<T, <Operation<K, V> as Read>::Cfg>, ) -> Result<Self, Error>

Returns an Immutable qmdb initialized from cfg. Any uncommitted log operations will be discarded and the state of the db will be as of the last committed operation.

pub async fn sync(&mut self) -> Result<(), Error>

Sync all database state to disk. While this isn’t necessary to ensure durability of committed operations, periodic invocation may reduce memory usage and the time required to recover the database on restart.

pub async fn destroy(self) -> Result<(), Error>

Destroy the db, removing all data from disk.

pub fn new_batch( &self, ) -> UnmerkleizedBatch<'_, E, K, V, H, T, Mmr<E, H::Digest>>

Create a new speculative batch of operations with this database as its parent.

pub async fn apply_batch( &mut self, batch: Changeset<K, H::Digest, V>, ) -> Result<Range<Location>, Error>

Apply a changeset to the database.

A changeset is only valid if the database has not been modified since the batch that produced it was created. Multiple batches can be forked from the same parent for speculative execution, but only one may be applied. Applying a stale changeset returns Error::StaleChangeset.

Returns the range of locations written.