Keyless

Struct Keyless 

pub struct Keyless<E: Storage + Clock + Metrics, V: VariableValue, H: Hasher> { /* private fields */ }
A keyless authenticated database for variable-length data.

Implementations

Source

impl<E: Storage + Clock + Metrics, V: VariableValue, H: Hasher> Keyless<E, V, H>

pub async fn get(&self, loc: Location) -> Result<Option<V>, Error>

Get the value at location loc in the database.

Errors

Returns Error::LocationOutOfBounds if loc >= self.bounds().await.end.

pub const fn last_commit_loc(&self) -> Location

Returns the location of the last commit.

pub async fn get_metadata(&self) -> Result<Option<V>, Error>

Get the metadata associated with the last commit.

pub async fn init(context: E, cfg: Config<V::Cfg>) -> Result<Self, Error>

Returns a Keyless qmdb initialized from cfg. Any uncommitted operations will be discarded and the state of the db will be as of the last committed operation.

pub fn root(&self) -> H::Digest

Return the root of the db.

pub async fn proof( &self, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<V>>), Error>

Generate and return:

  1. a proof of all operations applied to the db in the range starting at (and including) location start_loc, and ending at the first of either:
    • the last operation performed, or
    • the operation max_ops from the start.
  2. the operations corresponding to the leaves in this range.
pub async fn historical_proof( &self, op_count: Location, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<V>>), Error>

Analogous to proof, but with respect to the state of the MMR when it had op_count operations.

Errors
pub async fn prune(&mut self, loc: Location) -> Result<(), Error>

Prune historical operations prior to loc. This does not affect the db’s root.

Errors
pub async fn sync(&mut self) -> Result<(), Error>

Sync all database state to disk. While this isn’t necessary to ensure durability of committed operations, periodic invocation may reduce memory usage and the time required to recover the database on restart.

pub async fn destroy(self) -> Result<(), Error>

Destroy the db, removing all data from disk.

pub fn new_batch(&self) -> UnmerkleizedBatch<'_, E, V, H, Mmr<E, H::Digest>>

Create a new speculative batch of operations with this database as its parent.

pub async fn apply_batch( &mut self, batch: Changeset<H::Digest, V>, ) -> Result<Range<Location>, Error>

Apply a changeset to the database.

A changeset is only valid if the database has not been modified since the batch that produced it was created. Multiple batches can be forked from the same parent for speculative execution, but only one may be applied. Applying a stale changeset returns Error::StaleChangeset.

Returns the range of locations written.