pub struct Keyless<E: Storage + Clock + Metrics, V: VariableValue, H: Hasher> { /* private fields */ }Expand description
A keyless authenticated database for variable-length data.
Implementations
Sourceimpl<E: Storage + Clock + Metrics, V: VariableValue, H: Hasher> Keyless<E, V, H>
impl<E: Storage + Clock + Metrics, V: VariableValue, H: Hasher> Keyless<E, V, H>
Sourcepub async fn get(&self, loc: Location) -> Result<Option<V>, Error>
pub async fn get(&self, loc: Location) -> Result<Option<V>, Error>
Get the value at location loc in the database.
Errors
Returns Error::LocationOutOfBounds if loc >= self.bounds().await.end.
Sourcepub const fn last_commit_loc(&self) -> Location
pub const fn last_commit_loc(&self) -> Location
Returns the location of the last commit.
Sourcepub async fn get_metadata(&self) -> Result<Option<V>, Error>
pub async fn get_metadata(&self) -> Result<Option<V>, Error>
Get the metadata associated with the last commit.
Sourcepub async fn init(context: E, cfg: Config<V::Cfg>) -> Result<Self, Error>
pub async fn init(context: E, cfg: Config<V::Cfg>) -> Result<Self, Error>
Returns a Keyless qmdb initialized from cfg. Any uncommitted operations will be discarded
and the state of the db will be as of the last committed operation.
Sourcepub async fn proof(
&self,
start_loc: Location,
max_ops: NonZeroU64,
) -> Result<(Proof<H::Digest>, Vec<Operation<V>>), Error>
pub async fn proof( &self, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<V>>), Error>
Generate and return:
- a proof of all operations applied to the db in the range starting at (and including)
location
start_loc, and ending at the first of either:- the last operation performed, or
- the operation
max_opsfrom the start.
- the operations corresponding to the leaves in this range.
Sourcepub async fn historical_proof(
&self,
op_count: Location,
start_loc: Location,
max_ops: NonZeroU64,
) -> Result<(Proof<H::Digest>, Vec<Operation<V>>), Error>
pub async fn historical_proof( &self, op_count: Location, start_loc: Location, max_ops: NonZeroU64, ) -> Result<(Proof<H::Digest>, Vec<Operation<V>>), Error>
Analogous to proof, but with respect to the state of the MMR when it had op_count
operations.
Errors
- Returns crate::mmr::Error::LocationOverflow if
op_countorstart_loc> crate::mmr::MAX_LOCATION. - Returns crate::mmr::Error::RangeOutOfBounds if
start_loc>=op_countorop_count> number of operations.
Sourcepub async fn prune(&mut self, loc: Location) -> Result<(), Error>
pub async fn prune(&mut self, loc: Location) -> Result<(), Error>
Prune historical operations prior to loc. This does not affect the db’s root.
Errors
- Returns Error::PruneBeyondMinRequired if
loc> last commit point. - Returns crate::mmr::Error::LocationOverflow if
loc> crate::mmr::MAX_LOCATION
Sourcepub async fn sync(&mut self) -> Result<(), Error>
pub async fn sync(&mut self) -> Result<(), Error>
Sync all database state to disk. While this isn’t necessary to ensure durability of committed operations, periodic invocation may reduce memory usage and the time required to recover the database on restart.
Sourcepub fn new_batch(&self) -> UnmerkleizedBatch<'_, E, V, H, Mmr<E, H::Digest>>
pub fn new_batch(&self) -> UnmerkleizedBatch<'_, E, V, H, Mmr<E, H::Digest>>
Create a new speculative batch of operations with this database as its parent.
Sourcepub async fn apply_batch(
&mut self,
batch: Changeset<H::Digest, V>,
) -> Result<Range<Location>, Error>
pub async fn apply_batch( &mut self, batch: Changeset<H::Digest, V>, ) -> Result<Range<Location>, Error>
Apply a changeset to the database.
A changeset is only valid if the database has not been modified since the
batch that produced it was created. Multiple batches can be forked from the
same parent for speculative execution, but only one may be applied. Applying
a stale changeset returns Error::StaleChangeset.
Returns the range of locations written.