pub struct Reader<E: Clock + Storage + Metrics, V: CodecShared> { /* private fields */ }Expand description
Reader handle for dequeuing and acknowledging items.
There should only be one reader per shared queue.
Implementations
Sourceimpl<E: Clock + Storage + Metrics, V: CodecShared> Reader<E, V>
impl<E: Clock + Storage + Metrics, V: CodecShared> Reader<E, V>
Sourcepub async fn recv(&mut self) -> Result<Option<(u64, V)>, Error>
pub async fn recv(&mut self) -> Result<Option<(u64, V)>, Error>
Receive the next unacknowledged item, waiting if necessary.
This method is designed for use with select!. It will:
- Return immediately if an unacked item is available
- Wait for the writer to enqueue new items if the queue is empty
- Return
Noneif the writer is dropped (no more items will arrive)
Errors
Returns an error if the underlying storage operation fails.
Sourcepub async fn try_recv(&mut self) -> Result<Option<(u64, V)>, Error>
pub async fn try_recv(&mut self) -> Result<Option<(u64, V)>, Error>
Try to dequeue the next unacknowledged item without waiting.
Returns None immediately if no unacked item is available.
Errors
Returns an error if the underlying storage operation fails.
Sourcepub async fn ack_floor(&self) -> u64
pub async fn ack_floor(&self) -> u64
See Queue::ack_floor.
Sourcepub async fn read_position(&self) -> u64
pub async fn read_position(&self) -> u64
See Queue::read_position.
Sourcepub async fn is_empty(&self) -> bool
pub async fn is_empty(&self) -> bool
See Queue::is_empty.
Sourcepub async fn reset(&self)
pub async fn reset(&self)
See Queue::reset.