Context

Trait Context 

pub trait Context:
    Storage
    + Clock
    + Metrics { }
A runtime context providing storage, timing, and metrics capabilities.

This is a convenience alias for the trait bound Storage + Clock + Metrics that appears on nearly every type in this crate.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors

Source

impl<T: Storage + Clock + Metrics> Context for T

Available on crate feature std and neither commonware_stability_GAMMA nor commonware_stability_DELTA nor commonware_stability_EPSILON nor commonware_stability_RESERVED only.