pub trait Context:
Storage
+ Clock
+ Metrics { }Expand description
A runtime context providing storage, timing, and metrics capabilities.
This is a convenience alias for the trait bound
Storage + Clock + Metrics that appears on nearly every type in this crate.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.